searchData={"items":[{"type":"module","title":"Orb","doc":"Write WebAssembly modules with Elixir.\n\nWebAssembly is a low-level language. You work with integers and floats, can perform operations on them like adding or multiplication, and then read and write those values to a block of memory. There’s no concept of a “string” or an “array”, let alone a “hash map” or “HTTP request”.\n\nThat’s where a library like Orb can help out. It takes full advantage of Elixir’s language features by becoming a compiler for WebAssembly. You can define WebAssembly modules in Elixir for “string” or “hash map”, and compose them together into a final module.\n\nThat WebAssembly module can then run in every major application environment: browsers, servers, the edge, and mobile devices like phones, tablets & laptops. This story is still being developed, but I believe like other web standards like JSON, HTML, and HTTP, that WebAssembly will become a first-class citizen on any platform. It’s Turing-complete, designed to be backwards compatible, fast, and works almost everywhere.","ref":"Orb.html"},{"type":"module","title":"Example - Orb","doc":"Let’s create a module that calculates the average of a set of numbers.\n\nWebAssembly modules can have state. Here will have two pieces of state: a total `count` and a running `tally`. These are stored as **globals**. (If you are familiar with object-oriented programming, you can think of them as instance variables).\n\nOur module will export two functions: `insert` and `calculate_mean`. These two functions will work with the `count` and `tally` globals.\n\n```elixir\ndefmodule CalculateMean do\n  use Orb\n\n  global do\n    @count 0\n    @tally 0\n  end\n\n  defw insert(element: I32) do\n    @count = @count + 1\n    @tally = @tally + element\n  end\n\n  defw calculate_mean(), I32 do\n    @tally / @count\n  end\nend\n```\n\nOne thing you’ll notice is that we must specify the type of function parameters and return values. Our `insert` function accepts a 32-bit integer, denoted using `I32`. It returns no value, while `calculate_mean` is annotated to return a 32-bit integer.\n\nWe get to write math with the intuitive `+` and `/` operators. Let’s see the same module without the magic: no math operators and without `@` conveniences for working with globals:\n\n```elixir\ndefmodule CalculateMean do\n  use Orb\n\n  I32.global(count: 0, tally: 0)\n\n  defw insert(element: I32) do\n    I32.add(global_get(:count), 1)\n    global_set(:count)\n    I32.add(global_get(:tally), element)\n    global_set(:tally)\n  end\n\n  defw calculate_mean(), I32 do\n    I32.div_s(global_get(:tally), global_get(:count))\n  end\nend\n```\n\nThis is the exact same logic as before. In fact, this is what the first version expands to. Orb adds “sugar syntax” to make authoring WebAssembly nicer, to make it feel like writing Elixir or Ruby.","ref":"Orb.html#module-example"},{"type":"module","title":"Functions - Orb","doc":"In Elixir you define functions publicly available outside the module with `def/1`, and functions private to the module with `defp/1`. Orb follows the same suffix convention with `defw/2` and `defwp/2`.\n\nConsumers of your WebAssembly module will only be able to call exported functions defined using `defw/2`. Making a function public in WebAssembly is known as “exporting”.","ref":"Orb.html#module-functions"},{"type":"module","title":"Stack based - Orb","doc":"While it looks like Elixir, there are some key differences between it and programs written in Orb. The first is that state is mutable. While immutability is one of the best features of Elixir, in WebAssembly variables are mutable because raw computer memory is mutable.\n\nThe second key difference is that WebAssembly is stack based. Every function has an implicit stack of values that you can push and pop from. This paradigm allows WebAssembly runtimes to efficiently optimize for raw CPU registers whilst not being platform specific.\n\nIn Elixir when you write:\n\n```elixir\ndef example() do\n  1\n  2\n  3\nend\n```\n\nThe first two lines with `1` and `2` are inert — they have no effect — and the result from the function is the last line `3`.\n\nIn WebAssembly / Orb when you write the same sort of thing:\n\n```elixir\ndefw example() do\n  1\n  2\n  3\nend\n```\n\nThen what’s happening is that we are pushing `1` onto the stack, then `2`, and then `3`. Now the stack has three items on it. Which will become our return value: a tuple of 3 integers. (Our function has no return type specified, so this will be an error if you attempted to compile the resulting module).\n\nSo the correct return type from this function would be a tuple of three integers:\n\n```elixir\ndefw example(), {I32, I32, I32} do\n  1\n  2\n  3\nend\n```\n\nIf you prefer, Orb allows you to be explicit with your stack pushes with `Orb.DSL.push/1`:\n\n```elixir\ndefw example(), {I32, I32, I32} do\n  push(1)\n  push(2)\n  push(3)\nend\n```\n\nYou can use the stack to unlock novel patterns, but for the most part Orb avoids the need to interact with it. It’s just something to keep in mind if you are used to lines of code with simple values not having any side effects.","ref":"Orb.html#module-stack-based"},{"type":"module","title":"Locals - Orb","doc":"Locals are variables that live for the lifetime of a function. They must be specified upfront with their type alongside the function’s definition, and are initialized to zero.\n\nHere we have two locals: `under?` and `over?`, both 32-bit integers. We can set their value and then read them again at the bottom of the function.\n\n```elixir\ndefmodule WithinRange do\n  use Orb\n\n  defw validate(num: I32), I32, under?: I32, over?: I32 do\n    under? = num   255\n\n    not (under? or over?)\n  end\nend\n```","ref":"Orb.html#module-locals"},{"type":"module","title":"Globals - Orb","doc":"Globals are like locals, but live for the duration of the entire running module’s life. Their initial type and value are specified upfront.\n\nGlobals by default are internal: nothing outside the module can see them. They can be exported to expose them to the outside world.\n\n```elixir\ndefmodule GlobalExample do\n  use Orb\n\n  global do # :mutable by default\n    @some_internal_global 99\n  end\n\n  global :readonly do\n    @some_internal_constant 99\n  end\n\n  global :export_readonly do\n    @some_public_constant 1001\n  end\n\n  global :export_mutable do\n    @some_public_variable 42\n  end\n\n  # You can define multiple globals at once:\n  global do\n    @magic_number_a 99\n    @magic_number_b 12\n    @magic_number_c -5\n  end\nend\n```\n\nYou can read or write to a global within `defw` using the `@` prefix:\n\n```elixir\ndefmodule Counter do\n  use Orb\n\n  global do\n    @counter 0\n  end\n\n  defw increment() do\n    @counter = @counter + 1\n  end\nend\n```\n\nWhen you use `Orb.global/1` an Elixir module attribute with the same name and initial value is also defined for you:application\n\n```elixir\ndefmodule DeepThought do\n  use Orb\n\n  global do\n    @meaning_of_life 42\n  end\n\n  def get_meaning_of_life_elixir() do\n    @meaning_of_life\n  end\n\n  defw get_meaning_of_life_wasm(), I32 do\n    @meaning_of_life\n  end\nend\n```","ref":"Orb.html#module-globals"},{"type":"module","title":"Memory - Orb","doc":"WebAssembly provides a buffer of memory when you need more than a handful global integers or floats. This is a contiguous array of random-access memory which you can freely read and write to.\n\n#","ref":"Orb.html#module-memory"},{"type":"module","title":"Pages - Orb","doc":"WebAssembly Memory comes in 64 KiB segments called pages. You use some multiple of these 64 KiB (64 * 1024 = 65,536 bytes) pages.\n\nBy default your module will have **no** memory, so you must specify how much memory you want upfront.\n\nHere’s an example with 16 pages (1 MiB) of memory:\n\n```elixir\ndefmodule Example do\n  use Orb\n\n  Memory.pages(16)\nend\n```\n\n#","ref":"Orb.html#module-pages"},{"type":"module","title":"Reading & writing memory - Orb","doc":"To read from memory, you can use the `Memory.load/2` function. This loads a value at the given memory address. Addresses are themselves 32-bit integers. This mean you can perform pointer arithmetic to calculate whatever address you need to access.\n\nHowever, this can prove unsafe as it’s easy to calculate the wrong address and corrupt your memory. For this reason, Orb provides higher level constructs for making working with memory pointers more pleasant, which are detailed later on.\n\n```elixir\ndefmodule Example do\n  use Orb\n\n  Memory.pages(1)\n\n  defw get_int32(), I32 do\n    Memory.load!(I32, 0x100)\n  end\n\n  defw set_int32(value: I32) do\n    Memory.store!(I32, 0x100, value)\n  end\nend\n```\n\n#","ref":"Orb.html#module-reading-writing-memory"},{"type":"module","title":"Initializing memory with data - Orb","doc":"You can populate the initial memory of your module using `Orb.Memory.initial_data!/2`. This accepts an memory offset and the string to write there.\n\n```elixir\ndefmodule MimeTypeDataExample do\n  use Orb\n\n  Memory.pages(1)\n\n  Memory.initial_data!(0x100, \"text/html\")\n  Memory.initial_data!(0x200, \"\"\"\n     \n     \n     Hello world \n    \"\"\")\n\n  defw get_mime_type(), I32 do\n    0x100\n  end\n\n  defw get_body(), I32 do\n    0x200\n  end\nend\n```\n\nHaving to manually allocate and remember each memory offset is a pain, so Orb provides conveniences which are detailed in the next section.","ref":"Orb.html#module-initializing-memory-with-data"},{"type":"module","title":"Strings constants - Orb","doc":"You can use constant strings with the `~S` sigil. These will be extracted as initial data definitions at the start of the WebAssembly module, and their memory offsets substituted in their place.\n\nEach string is packed together for maximum efficiency of memory space. Strings are deduplicated, so you can use the same string constant multiple times and a single allocation will be made.\n\nString constants in Orb are nul-terminated.\n\n```elixir\ndefmodule MimeTypeStringExample do\n  use Orb\n\n  Memory.pages(1)\n\n  defw get_mime_type(), I32 do\n    ~S\"text/html\"\n  end\n\n  defw get_body(), I32 do\n    ~S\"\"\"\n     \n     \n     Hello world \n    \"\"\"\n  end\nend\n```","ref":"Orb.html#module-strings-constants"},{"type":"module","title":"Control flow - Orb","doc":"Orb supports control flow with `if`, `block`, and `loop` statements.\n\n#","ref":"Orb.html#module-control-flow"},{"type":"module","title":"If statements - Orb","doc":"If you want to run logic conditionally, use an `if` statement.\n\n```elixir\nif @party_mode? do\n  music_volume = 100\nend\n```\n\nYou can add an `else` clause:\n\n```elixir\nif @party_mode? do\n  music_volume = 100\nelse\n  music_volume = 30\nend\n```\n\nIf you want a ternary operator (e.g. to map from one value to another), you can use `Orb.I32.when?/2` instead:\n\n```elixir\nmusic_volume = I32.when? @party_mode? do\n  100\nelse\n  30\nend\n```\n\nThese can be written on single line too:\n\n```elixir\nmusic_volume = I32.when?(@party_mode?, do: 100, else: 30)\n```\n\n#","ref":"Orb.html#module-if-statements"},{"type":"module","title":"Loops - Orb","doc":"Loops look like the familiar construct in other languages like JavaScript, with two key differences: each loop has a name, and loops by default stop unless you explicitly tell them to continue.\n\n```elixir\ni = 0\nloop CountUp do\n  i = i + 1\n\n  CountUp.continue(if: i < 10)\nend\n```\n\nEach loop is named, so if you nest them you can specify which particular one to continue.\n\n```elixir\ntotal_weeks = 10\nweekday_count = 7\nweek = 0\nweekday = 0\nloop Weeks do\n  loop Weekdays do\n    # Do something here with week and weekday\n\n    weekday = weekday + 1\n    Weekdays.continue(if: weekday < weekday_count)\n  end\n\n  week = week + 1\n  Weeks.continue(if: week < total_weeks)\nend\n```\n\n##","ref":"Orb.html#module-loops"},{"type":"module","title":"Iterators - Orb","doc":"Iterators are an upcoming feature, currently part of SilverOrb that will hopefully become part of Orb itself.\n\n#","ref":"Orb.html#module-iterators"},{"type":"module","title":"Blocks - Orb","doc":"Blocks provide a structured way to skip code.\n\n```elixir\nControl.block Validate do\n  Validate.break(if: i < 0)\n\n  # Do something with i\nend\n```\n\nBlocks can have a type.\n\n```elixir\nControl.block Double, I32 do\n  if i < 0 do\n    push(0)\n    Double.break()\n  end\n\n  push(i * 2)\nend\n```","ref":"Orb.html#module-blocks"},{"type":"module","title":"Calling other functions - Orb","doc":"When you use `defw`, a corresponding Elixir function is defined for you using `def`.\n\n```elixir\ndefw magic_number(), I32 do\n  42\nend\n```\n\n```elixir\ndefw some_example(), n: I32 do\n  n = magic_number()\nend\n```\n\nYou can also use `Orb.DSL.typed_call/3` to manually call functions defined within your module. Currently, the parameters are not checked, so you must ensure you are calling with the correct arity and types.\n\n```elixir\nchar = typed_call(I32, :encode_html_char, char)\n```","ref":"Orb.html#module-calling-other-functions"},{"type":"module","title":"Composing modules with `Orb.include/1` - Orb","doc":"The WebAssembly functions from one module reused in another using `Orb.include/1`.\n\nHere’s an example of module A’s square function being included by module B:\n\n```elixir\ndefmodule A do\n  use Orb\n\n  defw square(n: I32), I32 do\n    n * n\n  end\nend\n\ndefmodule B do\n  use Orb\n\n  # Copies all WebAssembly functions defined in A into this module.\n  Orb.include(A)\n\n  defw example(n: I32), I32 do\n    # Now we can call functions on A.\n    A.square(42)\n  end\nend\n```","ref":"Orb.html#module-composing-modules-with-orb-include-1"},{"type":"module","title":"Importing with `Orb.importw/2` - Orb","doc":"Your running WebAssembly module can interact with the outside world by importing globals and functions.","ref":"Orb.html#module-importing-with-orb-importw-2"},{"type":"module","title":"Use Elixir features - Orb","doc":"- Piping","ref":"Orb.html#module-use-elixir-features"},{"type":"module","title":"Inline - Orb","doc":"- `inline do:`\n  - Module attributes\n  - `wasm do:`\n- `inline for`\n\n#","ref":"Orb.html#module-inline"},{"type":"module","title":"Custom types with `Access` - Orb","doc":"TODO: extract this into its own section.","ref":"Orb.html#module-custom-types-with-access"},{"type":"module","title":"Define your own functions and macros - Orb","doc":"","ref":"Orb.html#module-define-your-own-functions-and-macros"},{"type":"module","title":"Hex packages - Orb","doc":"- SilverOrb\n    - String builder\n- GoldenOrb","ref":"Orb.html#module-hex-packages"},{"type":"module","title":"Running your module - Orb","doc":"","ref":"Orb.html#module-running-your-module"},{"type":"macro","title":"Orb.__append_body/2","doc":"Enter WebAssembly.","ref":"Orb.html#__append_body/2"},{"type":"macro","title":"Orb.functype/2","doc":"","ref":"Orb.html#functype/2"},{"type":"macro","title":"Orb.global/2","doc":"Declare WebAssembly globals.\n\n`mode` can be :readonly, :mutable, :export_readonly, or :export_mutable. The default is :mutable.","ref":"Orb.html#global/2"},{"type":"macro","title":"Examples - Orb.global/2","doc":"```elixir\ndefmodule GlobalExample do\n  use Orb\n\n  global do # :mutable by default\n    @some_internal_global 99\n  end\n\n  global :readonly do\n    @some_internal_constant 99\n  end\n\n  global :export_readonly do\n    @some_public_constant 1001\n  end\n\n  global :export_mutable do\n    @some_public_variable 42\n  end\n\n  # You can define multiple globals at once:\n  global do\n    @magic_number_a 99\n    @magic_number_b 12\n    @magic_number_c -5\n  end\nend\n```","ref":"Orb.html#global/2-examples"},{"type":"macro","title":"Orb.importw/2","doc":"","ref":"Orb.html#importw/2"},{"type":"macro","title":"Orb.include/1","doc":"Copy WebAssembly functions from one module into the current module.","ref":"Orb.html#include/1"},{"type":"macro","title":"Examples - Orb.include/1","doc":"```elixir\ndefmodule Math do\n  use Orb\n\n  defw square(n: I32), I32 do\n    n * n\n  end\nend\n\ndefmodule SomeOtherModule do\n  use Orb\n\n  Orb.include(Math)\n\n  defw magic(), I32 do\n    Math.square(3)\n  end\nend\n```","ref":"Orb.html#include/1-examples"},{"type":"macro","title":"Orb.set_func_prefix/1","doc":"Adds a prefix to all functions within this module. This namespacing helps avoid names from clashing.","ref":"Orb.html#set_func_prefix/1"},{"type":"macro","title":"Orb.snippet/3","doc":"Declare a snippet of Orb AST for reuse. Enables DSL, with additions from `mode`.","ref":"Orb.html#snippet/3"},{"type":"function","title":"Orb.to_wasm/1","doc":"Convert Orb AST into WebAssembly binary format.","ref":"Orb.html#to_wasm/1"},{"type":"function","title":"Orb.to_wat/1","doc":"Convert Orb AST into WebAssembly text format.","ref":"Orb.html#to_wat/1"},{"type":"macro","title":"Orb.types/1","doc":"","ref":"Orb.html#types/1"},{"type":"module","title":"Orb.Block.Branch","doc":"","ref":"Orb.Block.Branch.html"},{"type":"module","title":"Orb.Constants.NulTerminatedString","doc":"","ref":"Orb.Constants.NulTerminatedString.html"},{"type":"function","title":"Orb.Constants.NulTerminatedString.empty/0","doc":"","ref":"Orb.Constants.NulTerminatedString.html#empty/0"},{"type":"module","title":"Orb.Control","doc":"","ref":"Orb.Control.html"},{"type":"macro","title":"Orb.Control.block/3","doc":"Declare a block, useful for structured control flow.","ref":"Orb.Control.html#block/3"},{"type":"function","title":"Orb.Control.break/1","doc":"Break from a block.","ref":"Orb.Control.html#break/1"},{"type":"function","title":"Orb.Control.break/2","doc":"Break from a block if a condition is true.","ref":"Orb.Control.html#break/2"},{"type":"function","title":"Orb.Control.return/0","doc":"","ref":"Orb.Control.html#return/0"},{"type":"module","title":"Orb.Control.Return","doc":"","ref":"Orb.Control.Return.html"},{"type":"function","title":"Orb.Control.Return.new/0","doc":"","ref":"Orb.Control.Return.html#new/0"},{"type":"function","title":"Orb.Control.Return.new/1","doc":"","ref":"Orb.Control.Return.html#new/1"},{"type":"behaviour","title":"Orb.CustomType","doc":"","ref":"Orb.CustomType.html"},{"type":"callback","title":"Orb.CustomType.byte_count/0","doc":"","ref":"Orb.CustomType.html#c:byte_count/0"},{"type":"callback","title":"Orb.CustomType.load_instruction/0","doc":"","ref":"Orb.CustomType.html#c:load_instruction/0"},{"type":"function","title":"Orb.CustomType.resolve!/1","doc":"","ref":"Orb.CustomType.html#resolve!/1"},{"type":"callback","title":"Orb.CustomType.wasm_type/0","doc":"","ref":"Orb.CustomType.html#c:wasm_type/0"},{"type":"module","title":"Orb.DSL","doc":"The main DSL which is imported automatically when you call `Orb.wasm/2`.","ref":"Orb.DSL.html"},{"type":"function","title":"Orb.DSL.assert!/1","doc":"Asserts a condition that _must_ be true, otherwise traps.","ref":"Orb.DSL.html#assert!/1"},{"type":"function","title":"Orb.DSL.call/1","doc":"Call local function `f`.","ref":"Orb.DSL.html#call/1"},{"type":"function","title":"Orb.DSL.call/2","doc":"Call local function `f`, passing arguments `args` when list, or single argument otherwise.","ref":"Orb.DSL.html#call/2"},{"type":"function","title":"Orb.DSL.call/3","doc":"Call local function `f`, passing arguments `a` & `b`.","ref":"Orb.DSL.html#call/3"},{"type":"function","title":"Orb.DSL.call/4","doc":"Call local function `f`, passing argument `a`, `b`, `c`.","ref":"Orb.DSL.html#call/4"},{"type":"function","title":"Orb.DSL.call/5","doc":"Call local function `f`, passing argument `a`, `b`, `c`, `d`.","ref":"Orb.DSL.html#call/5"},{"type":"function","title":"Orb.DSL.call/6","doc":"Call local function `f`, passing argument `a`, `b`, `c`, `d`, `e`.","ref":"Orb.DSL.html#call/6"},{"type":"function","title":"Orb.DSL.call/7","doc":"Call local function `f`, passing argument `a`, `b`, `c`, `d`, `e`, `f`.","ref":"Orb.DSL.html#call/7"},{"type":"function","title":"Orb.DSL.const/1","doc":"Declare a constant string, which will be extracted to the top of the module, and its address substituted in place.","ref":"Orb.DSL.html#const/1"},{"type":"function","title":"Orb.DSL.do_snippet/2","doc":"","ref":"Orb.DSL.html#do_snippet/2"},{"type":"function","title":"Orb.DSL.drop/0","doc":"Drop the last pushed value from the stack.","ref":"Orb.DSL.html#drop/0"},{"type":"function","title":"Orb.DSL.drop/1","doc":"Execute the passed expression, but ignore its result by immediately dropping its value.","ref":"Orb.DSL.html#drop/1"},{"type":"function","title":"Orb.DSL.export/2","doc":"","ref":"Orb.DSL.html#export/2"},{"type":"macro","title":"Orb.DSL.func/2","doc":"","ref":"Orb.DSL.html#func/2"},{"type":"macro","title":"Orb.DSL.func/3","doc":"","ref":"Orb.DSL.html#func/3"},{"type":"macro","title":"Orb.DSL.func/4","doc":"","ref":"Orb.DSL.html#func/4"},{"type":"function","title":"Orb.DSL.funcp/1","doc":"","ref":"Orb.DSL.html#funcp/1"},{"type":"macro","title":"Orb.DSL.funcp/2","doc":"","ref":"Orb.DSL.html#funcp/2"},{"type":"macro","title":"Orb.DSL.funcp/3","doc":"","ref":"Orb.DSL.html#funcp/3"},{"type":"macro","title":"Orb.DSL.funcp/4","doc":"","ref":"Orb.DSL.html#funcp/4"},{"type":"function","title":"Orb.DSL.global_get/1","doc":"","ref":"Orb.DSL.html#global_get/1"},{"type":"function","title":"Orb.DSL.global_set/1","doc":"","ref":"Orb.DSL.html#global_set/1"},{"type":"function","title":"Orb.DSL.i32/1","doc":"","ref":"Orb.DSL.html#i32/1"},{"type":"function","title":"Orb.DSL.i64/1","doc":"","ref":"Orb.DSL.html#i64/1"},{"type":"macro","title":"Orb.DSL.inline/1","doc":"Run code at compile-time.","ref":"Orb.DSL.html#inline/1"},{"type":"macro","title":"Orb.DSL.inline/2","doc":"Run Elixir for-comprehension at compile-time.","ref":"Orb.DSL.html#inline/2"},{"type":"function","title":"Orb.DSL.local_set/1","doc":"","ref":"Orb.DSL.html#local_set/1"},{"type":"macro","title":"Orb.DSL.loop/2","doc":"Declare a loop that iterates through a source.","ref":"Orb.DSL.html#loop/2"},{"type":"macro","title":"Orb.DSL.loop/3","doc":"Declare a loop.","ref":"Orb.DSL.html#loop/3"},{"type":"function","title":"Orb.DSL.mut!/1","doc":"","ref":"Orb.DSL.html#mut!/1"},{"type":"function","title":"Orb.DSL.nop/0","doc":"A no-op instruction.\n\nSee https://en.wikipedia.org/wiki/NOP_(code)","ref":"Orb.DSL.html#nop/0"},{"type":"function","title":"Orb.DSL.push/1","doc":"Pushes a value onto the current stack.","ref":"Orb.DSL.html#push/1"},{"type":"function","title":"Orb.DSL.push/2","doc":"Push value then run the block. Useful for when you mutate a variable but want its previous value.","ref":"Orb.DSL.html#push/2"},{"type":"function","title":"Orb.DSL.raw_wat/1","doc":"For when there’s a language feature of WebAssembly that Orb doesn’t provide. Please file an issue if there’s something you wish existed. https://github.com/RoyalIcing/Orb/issues","ref":"Orb.DSL.html#raw_wat/1"},{"type":"function","title":"Orb.DSL.return/0","doc":"Return from a function. You may wish to `push/1` values before returning.","ref":"Orb.DSL.html#return/0"},{"type":"function","title":"Orb.DSL.return/1","doc":"Return from a function if a condition is true.","ref":"Orb.DSL.html#return/1"},{"type":"function","title":"Orb.DSL.return/2","doc":"Return from a function with the provided value only if a condition is true.","ref":"Orb.DSL.html#return/2"},{"type":"function","title":"Orb.DSL.typed_call/3","doc":"","ref":"Orb.DSL.html#typed_call/3"},{"type":"function","title":"Orb.DSL.unreachable!/0","doc":"Denote a point in code that should not be reachable. Traps.\n\nUseful for exhaustive conditionals or code you know will not execute.","ref":"Orb.DSL.html#unreachable!/0"},{"type":"macro","title":"Orb.DSL.wasm/2","doc":"Used to bring back wasm-mode when inside `inline/1`.","ref":"Orb.DSL.html#wasm/2"},{"type":"module","title":"Orb.DefwDSL","doc":"","ref":"Orb.DefwDSL.html"},{"type":"macro","title":"Orb.DefwDSL.defw/2","doc":"","ref":"Orb.DefwDSL.html#defw/2"},{"type":"macro","title":"Orb.DefwDSL.defw/3","doc":"","ref":"Orb.DefwDSL.html#defw/3"},{"type":"macro","title":"Orb.DefwDSL.defw/4","doc":"","ref":"Orb.DefwDSL.html#defw/4"},{"type":"macro","title":"Orb.DefwDSL.defwp/2","doc":"","ref":"Orb.DefwDSL.html#defwp/2"},{"type":"macro","title":"Orb.DefwDSL.defwp/3","doc":"","ref":"Orb.DefwDSL.html#defwp/3"},{"type":"macro","title":"Orb.DefwDSL.defwp/4","doc":"","ref":"Orb.DefwDSL.html#defwp/4"},{"type":"macro","title":"Orb.DefwDSL.wasm_mode/1","doc":"","ref":"Orb.DefwDSL.html#wasm_mode/1"},{"type":"module","title":"Orb.F32.DSL","doc":"32-bit float operators.","ref":"Orb.F32.DSL.html"},{"type":"function","title":"Orb.F32.DSL.*/2","doc":"","ref":"Orb.F32.DSL.html#*/2"},{"type":"function","title":"Orb.F32.DSL.+/2","doc":"","ref":"Orb.F32.DSL.html#+/2"},{"type":"function","title":"Orb.F32.DSL.-/2","doc":"","ref":"Orb.F32.DSL.html#-/2"},{"type":"function","title":"Orb.F32.DSL.//2","doc":"","ref":"Orb.F32.DSL.html#//2"},{"type":"function","title":"Orb.F32.DSL.!=/2","doc":"","ref":"Orb.F32.DSL.html#!=/2"},{"type":"function","title":"Orb.F32.DSL.!==/2","doc":"","ref":"Orb.F32.DSL.html#!==/2"},{"type":"function","title":"Orb.F32.DSL.</2","doc":"","ref":"Orb.F32.DSL.html#%3C/2"},{"type":"function","title":"Orb.F32.DSL.<=/2","doc":"","ref":"Orb.F32.DSL.html#%3C=/2"},{"type":"function","title":"Orb.F32.DSL.==/2","doc":"","ref":"Orb.F32.DSL.html#==/2"},{"type":"function","title":"Orb.F32.DSL.===/2","doc":"","ref":"Orb.F32.DSL.html#===/2"},{"type":"function","title":"Orb.F32.DSL.>/2","doc":"","ref":"Orb.F32.DSL.html#%3E/2"},{"type":"function","title":"Orb.F32.DSL.>=/2","doc":"","ref":"Orb.F32.DSL.html#%3E=/2"},{"type":"module","title":"Orb.Global.DSL","doc":"Adds @global_name support","ref":"Orb.Global.DSL.html"},{"type":"macro","title":"Orb.Global.DSL.@/1","doc":"","ref":"Orb.Global.DSL.html#@/1"},{"type":"module","title":"Orb.Global.Declare","doc":"","ref":"Orb.Global.Declare.html"},{"type":"module","title":"Orb.Global.Declare.DeclareDSL","doc":"","ref":"Orb.Global.Declare.DeclareDSL.html"},{"type":"macro","title":"Orb.Global.Declare.DeclareDSL.@/1","doc":"","ref":"Orb.Global.Declare.DeclareDSL.html#@/1"},{"type":"macro","title":"Orb.Global.Declare.DeclareDSL.register_global/2","doc":"","ref":"Orb.Global.Declare.DeclareDSL.html#register_global/2"},{"type":"module","title":"Orb.Global.Declare.ExportMutableDSL","doc":"","ref":"Orb.Global.Declare.ExportMutableDSL.html"},{"type":"macro","title":"Orb.Global.Declare.ExportMutableDSL.@/1","doc":"","ref":"Orb.Global.Declare.ExportMutableDSL.html#@/1"},{"type":"module","title":"Orb.Global.Declare.ExportReadonlyDSL","doc":"","ref":"Orb.Global.Declare.ExportReadonlyDSL.html"},{"type":"macro","title":"Orb.Global.Declare.ExportReadonlyDSL.@/1","doc":"","ref":"Orb.Global.Declare.ExportReadonlyDSL.html#@/1"},{"type":"module","title":"Orb.Global.Declare.MutableDSL","doc":"","ref":"Orb.Global.Declare.MutableDSL.html"},{"type":"macro","title":"Orb.Global.Declare.MutableDSL.@/1","doc":"","ref":"Orb.Global.Declare.MutableDSL.html#@/1"},{"type":"module","title":"Orb.Global.Declare.ReadonlyDSL","doc":"","ref":"Orb.Global.Declare.ReadonlyDSL.html"},{"type":"macro","title":"Orb.Global.Declare.ReadonlyDSL.@/1","doc":"","ref":"Orb.Global.Declare.ReadonlyDSL.html#@/1"},{"type":"module","title":"Orb.I32","doc":"Type for 32-bit integer.","ref":"Orb.I32.html"},{"type":"function","title":"Orb.I32.add/2","doc":"","ref":"Orb.I32.html#add/2"},{"type":"macro","title":"Orb.I32.attr_writer/1","doc":"","ref":"Orb.I32.html#attr_writer/1"},{"type":"macro","title":"Orb.I32.attr_writer/2","doc":"","ref":"Orb.I32.html#attr_writer/2"},{"type":"function","title":"Orb.I32.band/2","doc":"","ref":"Orb.I32.html#band/2"},{"type":"function","title":"Orb.I32.clz/1","doc":"","ref":"Orb.I32.html#clz/1"},{"type":"macro","title":"Orb.I32.cond/1","doc":"","ref":"Orb.I32.html#cond/1"},{"type":"function","title":"Orb.I32.const/1","doc":"","ref":"Orb.I32.html#const/1"},{"type":"function","title":"Orb.I32.ctz/1","doc":"","ref":"Orb.I32.html#ctz/1"},{"type":"function","title":"Orb.I32.div_s/2","doc":"","ref":"Orb.I32.html#div_s/2"},{"type":"function","title":"Orb.I32.div_u/2","doc":"","ref":"Orb.I32.html#div_u/2"},{"type":"macro","title":"Orb.I32.enum/2","doc":"","ref":"Orb.I32.html#enum/2"},{"type":"function","title":"Orb.I32.eq/2","doc":"","ref":"Orb.I32.html#eq/2"},{"type":"function","title":"Orb.I32.eqz/1","doc":"","ref":"Orb.I32.html#eqz/1"},{"type":"macro","title":"Orb.I32.export_enum/2","doc":"","ref":"Orb.I32.html#export_enum/2"},{"type":"macro","title":"Orb.I32.export_global/2","doc":"","ref":"Orb.I32.html#export_global/2"},{"type":"function","title":"Orb.I32.from_4_byte_ascii/1","doc":"Converts an ASCII little-endian 4-byte string to an 32-bit integer.","ref":"Orb.I32.html#from_4_byte_ascii/1"},{"type":"function","title":"Orb.I32.ge_s/2","doc":"","ref":"Orb.I32.html#ge_s/2"},{"type":"function","title":"Orb.I32.ge_u/2","doc":"","ref":"Orb.I32.html#ge_u/2"},{"type":"macro","title":"Orb.I32.global/2","doc":"","ref":"Orb.I32.html#global/2"},{"type":"function","title":"Orb.I32.gt_s/2","doc":"","ref":"Orb.I32.html#gt_s/2"},{"type":"function","title":"Orb.I32.gt_u/2","doc":"","ref":"Orb.I32.html#gt_u/2"},{"type":"function","title":"Orb.I32.in?/2","doc":"","ref":"Orb.I32.html#in?/2"},{"type":"function","title":"Orb.I32.le_s/2","doc":"","ref":"Orb.I32.html#le_s/2"},{"type":"function","title":"Orb.I32.le_u/2","doc":"","ref":"Orb.I32.html#le_u/2"},{"type":"function","title":"Orb.I32.lt_s/2","doc":"","ref":"Orb.I32.html#lt_s/2"},{"type":"function","title":"Orb.I32.lt_u/2","doc":"","ref":"Orb.I32.html#lt_u/2"},{"type":"macro","title":"Orb.I32.match/2","doc":"","ref":"Orb.I32.html#match/2"},{"type":"function","title":"Orb.I32.mul/2","doc":"","ref":"Orb.I32.html#mul/2"},{"type":"function","title":"Orb.I32.ne/2","doc":"","ref":"Orb.I32.html#ne/2"},{"type":"function","title":"Orb.I32.or/2","doc":"","ref":"Orb.I32.html#or/2"},{"type":"function","title":"Orb.I32.popcnt/1","doc":"","ref":"Orb.I32.html#popcnt/1"},{"type":"function","title":"Orb.I32.rem_s/2","doc":"","ref":"Orb.I32.html#rem_s/2"},{"type":"function","title":"Orb.I32.rem_u/2","doc":"","ref":"Orb.I32.html#rem_u/2"},{"type":"function","title":"Orb.I32.rotl/2","doc":"","ref":"Orb.I32.html#rotl/2"},{"type":"function","title":"Orb.I32.rotr/2","doc":"","ref":"Orb.I32.html#rotr/2"},{"type":"function","title":"Orb.I32.select/2","doc":"","ref":"Orb.I32.html#select/2"},{"type":"function","title":"Orb.I32.shl/2","doc":"","ref":"Orb.I32.html#shl/2"},{"type":"function","title":"Orb.I32.shr_s/2","doc":"","ref":"Orb.I32.html#shr_s/2"},{"type":"function","title":"Orb.I32.shr_u/2","doc":"","ref":"Orb.I32.html#shr_u/2"},{"type":"function","title":"Orb.I32.sub/2","doc":"","ref":"Orb.I32.html#sub/2"},{"type":"function","title":"Orb.I32.sum!/1","doc":"","ref":"Orb.I32.html#sum!/1"},{"type":"function","title":"Orb.I32.trunc_f32_s/1","doc":"","ref":"Orb.I32.html#trunc_f32_s/1"},{"type":"function","title":"Orb.I32.trunc_f32_u/1","doc":"","ref":"Orb.I32.html#trunc_f32_u/1"},{"type":"function","title":"Orb.I32.trunc_f64_s/1","doc":"","ref":"Orb.I32.html#trunc_f64_s/1"},{"type":"function","title":"Orb.I32.trunc_f64_u/1","doc":"","ref":"Orb.I32.html#trunc_f64_u/1"},{"type":"function","title":"Orb.I32.wasm_type/0","doc":"","ref":"Orb.I32.html#wasm_type/0"},{"type":"macro","title":"Orb.I32.when?/2","doc":"","ref":"Orb.I32.html#when?/2"},{"type":"function","title":"Orb.I32.wrap_i64/1","doc":"","ref":"Orb.I32.html#wrap_i64/1"},{"type":"function","title":"Orb.I32.xor/2","doc":"","ref":"Orb.I32.html#xor/2"},{"type":"module","title":"Orb.I32.DSL","doc":"The operators that work consistently signed or unsigned.\n\nSigned specific: `Orb.S32.DSL`\n\nUnsigned specific: `Orb.U32.DSL`","ref":"Orb.I32.DSL.html"},{"type":"function","title":"Orb.I32.DSL.&&&/2","doc":"","ref":"Orb.I32.DSL.html#&&&/2"},{"type":"function","title":"Orb.I32.DSL.*/2","doc":"","ref":"Orb.I32.DSL.html#*/2"},{"type":"function","title":"Orb.I32.DSL.+/2","doc":"","ref":"Orb.I32.DSL.html#+/2"},{"type":"function","title":"Orb.I32.DSL.-/2","doc":"","ref":"Orb.I32.DSL.html#-/2"},{"type":"function","title":"Orb.I32.DSL.!=/2","doc":"","ref":"Orb.I32.DSL.html#!=/2"},{"type":"function","title":"Orb.I32.DSL.!==/2","doc":"","ref":"Orb.I32.DSL.html#!==/2"},{"type":"function","title":"Orb.I32.DSL.<<</2","doc":"","ref":"Orb.I32.DSL.html#%3C%3C%3C/2"},{"type":"function","title":"Orb.I32.DSL.==/2","doc":"","ref":"Orb.I32.DSL.html#==/2"},{"type":"function","title":"Orb.I32.DSL.===/2","doc":"","ref":"Orb.I32.DSL.html#===/2"},{"type":"function","title":"Orb.I32.DSL.not/1","doc":"","ref":"Orb.I32.DSL.html#not/1"},{"type":"function","title":"Orb.I32.DSL.or/2","doc":"","ref":"Orb.I32.DSL.html#or/2"},{"type":"function","title":"Orb.I32.DSL.|||/2","doc":"","ref":"Orb.I32.DSL.html#%7C%7C%7C/2"},{"type":"module","title":"Orb.I32.String","doc":"Custom type for a nul-terminated ASCII string.","ref":"Orb.I32.String.html"},{"type":"function","title":"Orb.I32.String.funcp/0","doc":"Import all WebAssembly functions from this module’s Orb definition.","ref":"Orb.I32.String.html#funcp/0"},{"type":"function","title":"Orb.I32.String.funcp/1","doc":"Import a specific WebAssembly function from this module’s Orb definition.","ref":"Orb.I32.String.html#funcp/1"},{"type":"macro","title":"Orb.I32.String.match/2","doc":"","ref":"Orb.I32.String.html#match/2"},{"type":"function","title":"Orb.I32.String.null/0","doc":"","ref":"Orb.I32.String.html#null/0"},{"type":"function","title":"Orb.I32.String.streq/2","doc":"","ref":"Orb.I32.String.html#streq/2"},{"type":"function","title":"Orb.I32.String.strlen/1","doc":"","ref":"Orb.I32.String.html#strlen/1"},{"type":"function","title":"Orb.I32.String.to_wat/0","doc":"Convert this module’s Orb definition to WebAssembly text (Wat) format.","ref":"Orb.I32.String.html#to_wat/0"},{"type":"module","title":"Orb.I32.U8","doc":"Type for unsigned 32-bit integer interpreted as an 8-bit byte.","ref":"Orb.I32.U8.html"},{"type":"module","title":"Orb.I32.U8.UnsafePointer","doc":"Custom `Orb.CustomType` for pointer to 8-bit integer (byte) in memory.","ref":"Orb.I32.U8.UnsafePointer.html"},{"type":"module","title":"Orb.I32.UnsafePointer","doc":"Custom `Orb.CustomType` for pointer to 32-bit integer in memory.","ref":"Orb.I32.UnsafePointer.html"},{"type":"module","title":"Orb.I64","doc":"Type for 64-bit integer.","ref":"Orb.I64.html"},{"type":"function","title":"Orb.I64.add/2","doc":"","ref":"Orb.I64.html#add/2"},{"type":"function","title":"Orb.I64.band/2","doc":"","ref":"Orb.I64.html#band/2"},{"type":"function","title":"Orb.I64.clz/1","doc":"","ref":"Orb.I64.html#clz/1"},{"type":"function","title":"Orb.I64.ctz/1","doc":"","ref":"Orb.I64.html#ctz/1"},{"type":"function","title":"Orb.I64.div_s/2","doc":"","ref":"Orb.I64.html#div_s/2"},{"type":"function","title":"Orb.I64.div_u/2","doc":"","ref":"Orb.I64.html#div_u/2"},{"type":"function","title":"Orb.I64.eq/2","doc":"","ref":"Orb.I64.html#eq/2"},{"type":"function","title":"Orb.I64.eqz/1","doc":"","ref":"Orb.I64.html#eqz/1"},{"type":"function","title":"Orb.I64.extend_i32_s/1","doc":"","ref":"Orb.I64.html#extend_i32_s/1"},{"type":"function","title":"Orb.I64.extend_i32_u/1","doc":"","ref":"Orb.I64.html#extend_i32_u/1"},{"type":"function","title":"Orb.I64.ge_s/2","doc":"","ref":"Orb.I64.html#ge_s/2"},{"type":"function","title":"Orb.I64.ge_u/2","doc":"","ref":"Orb.I64.html#ge_u/2"},{"type":"function","title":"Orb.I64.gt_s/2","doc":"","ref":"Orb.I64.html#gt_s/2"},{"type":"function","title":"Orb.I64.gt_u/2","doc":"","ref":"Orb.I64.html#gt_u/2"},{"type":"function","title":"Orb.I64.le_s/2","doc":"","ref":"Orb.I64.html#le_s/2"},{"type":"function","title":"Orb.I64.le_u/2","doc":"","ref":"Orb.I64.html#le_u/2"},{"type":"function","title":"Orb.I64.lt_s/2","doc":"","ref":"Orb.I64.html#lt_s/2"},{"type":"function","title":"Orb.I64.lt_u/2","doc":"","ref":"Orb.I64.html#lt_u/2"},{"type":"function","title":"Orb.I64.mul/2","doc":"","ref":"Orb.I64.html#mul/2"},{"type":"function","title":"Orb.I64.ne/2","doc":"","ref":"Orb.I64.html#ne/2"},{"type":"function","title":"Orb.I64.or/2","doc":"","ref":"Orb.I64.html#or/2"},{"type":"function","title":"Orb.I64.popcnt/1","doc":"","ref":"Orb.I64.html#popcnt/1"},{"type":"function","title":"Orb.I64.rem_s/2","doc":"","ref":"Orb.I64.html#rem_s/2"},{"type":"function","title":"Orb.I64.rem_u/2","doc":"","ref":"Orb.I64.html#rem_u/2"},{"type":"function","title":"Orb.I64.rotl/2","doc":"","ref":"Orb.I64.html#rotl/2"},{"type":"function","title":"Orb.I64.rotr/2","doc":"","ref":"Orb.I64.html#rotr/2"},{"type":"function","title":"Orb.I64.shl/2","doc":"","ref":"Orb.I64.html#shl/2"},{"type":"function","title":"Orb.I64.shr_s/2","doc":"","ref":"Orb.I64.html#shr_s/2"},{"type":"function","title":"Orb.I64.shr_u/2","doc":"","ref":"Orb.I64.html#shr_u/2"},{"type":"function","title":"Orb.I64.sub/2","doc":"","ref":"Orb.I64.html#sub/2"},{"type":"function","title":"Orb.I64.trunc_f32_s/1","doc":"","ref":"Orb.I64.html#trunc_f32_s/1"},{"type":"function","title":"Orb.I64.trunc_f32_u/1","doc":"","ref":"Orb.I64.html#trunc_f32_u/1"},{"type":"function","title":"Orb.I64.trunc_f64_s/1","doc":"","ref":"Orb.I64.html#trunc_f64_s/1"},{"type":"function","title":"Orb.I64.trunc_f64_u/1","doc":"","ref":"Orb.I64.html#trunc_f64_u/1"},{"type":"function","title":"Orb.I64.xor/2","doc":"","ref":"Orb.I64.html#xor/2"},{"type":"module","title":"Orb.I64.DSL","doc":"The operators that work consistently signed or unsigned.\n\nSigned specific: `Orb.S64.DSL`\n\nUnsigned specific: `Orb.U64.DSL`","ref":"Orb.I64.DSL.html"},{"type":"function","title":"Orb.I64.DSL.&&&/2","doc":"","ref":"Orb.I64.DSL.html#&&&/2"},{"type":"function","title":"Orb.I64.DSL.*/2","doc":"","ref":"Orb.I64.DSL.html#*/2"},{"type":"function","title":"Orb.I64.DSL.+/2","doc":"","ref":"Orb.I64.DSL.html#+/2"},{"type":"function","title":"Orb.I64.DSL.-/2","doc":"","ref":"Orb.I64.DSL.html#-/2"},{"type":"function","title":"Orb.I64.DSL.!=/2","doc":"","ref":"Orb.I64.DSL.html#!=/2"},{"type":"function","title":"Orb.I64.DSL.!==/2","doc":"","ref":"Orb.I64.DSL.html#!==/2"},{"type":"function","title":"Orb.I64.DSL.<<</2","doc":"","ref":"Orb.I64.DSL.html#%3C%3C%3C/2"},{"type":"function","title":"Orb.I64.DSL.==/2","doc":"","ref":"Orb.I64.DSL.html#==/2"},{"type":"function","title":"Orb.I64.DSL.===/2","doc":"","ref":"Orb.I64.DSL.html#===/2"},{"type":"function","title":"Orb.I64.DSL.not/1","doc":"","ref":"Orb.I64.DSL.html#not/1"},{"type":"function","title":"Orb.I64.DSL.or/2","doc":"","ref":"Orb.I64.DSL.html#or/2"},{"type":"function","title":"Orb.I64.DSL.|||/2","doc":"","ref":"Orb.I64.DSL.html#%7C%7C%7C/2"},{"type":"module","title":"Orb.I64.Signed.DSL","doc":"Signed 32-bit integer operators.","ref":"Orb.I64.Signed.DSL.html"},{"type":"function","title":"Orb.I64.Signed.DSL.//2","doc":"","ref":"Orb.I64.Signed.DSL.html#//2"},{"type":"function","title":"Orb.I64.Signed.DSL.</2","doc":"","ref":"Orb.I64.Signed.DSL.html#%3C/2"},{"type":"function","title":"Orb.I64.Signed.DSL.<=/2","doc":"","ref":"Orb.I64.Signed.DSL.html#%3C=/2"},{"type":"function","title":"Orb.I64.Signed.DSL.>/2","doc":"","ref":"Orb.I64.Signed.DSL.html#%3E/2"},{"type":"function","title":"Orb.I64.Signed.DSL.>=/2","doc":"","ref":"Orb.I64.Signed.DSL.html#%3E=/2"},{"type":"function","title":"Orb.I64.Signed.DSL.>>>/2","doc":"","ref":"Orb.I64.Signed.DSL.html#%3E%3E%3E/2"},{"type":"module","title":"Orb.Import.DSL","doc":"","ref":"Orb.Import.DSL.html"},{"type":"macro","title":"Orb.Import.DSL.defw/2","doc":"","ref":"Orb.Import.DSL.html#defw/2"},{"type":"module","title":"Orb.Instruction","doc":"","ref":"Orb.Instruction.html"},{"type":"function","title":"Orb.Instruction.call/1","doc":"","ref":"Orb.Instruction.html#call/1"},{"type":"function","title":"Orb.Instruction.call/2","doc":"","ref":"Orb.Instruction.html#call/2"},{"type":"function","title":"Orb.Instruction.f32/1","doc":"","ref":"Orb.Instruction.html#f32/1"},{"type":"function","title":"Orb.Instruction.f32/2","doc":"","ref":"Orb.Instruction.html#f32/2"},{"type":"function","title":"Orb.Instruction.f32/3","doc":"","ref":"Orb.Instruction.html#f32/3"},{"type":"function","title":"Orb.Instruction.global_get/2","doc":"","ref":"Orb.Instruction.html#global_get/2"},{"type":"function","title":"Orb.Instruction.global_set/3","doc":"","ref":"Orb.Instruction.html#global_set/3"},{"type":"function","title":"Orb.Instruction.i32/1","doc":"","ref":"Orb.Instruction.html#i32/1"},{"type":"function","title":"Orb.Instruction.i32/2","doc":"","ref":"Orb.Instruction.html#i32/2"},{"type":"function","title":"Orb.Instruction.i32/3","doc":"","ref":"Orb.Instruction.html#i32/3"},{"type":"function","title":"Orb.Instruction.i64/1","doc":"","ref":"Orb.Instruction.html#i64/1"},{"type":"function","title":"Orb.Instruction.i64/2","doc":"","ref":"Orb.Instruction.html#i64/2"},{"type":"function","title":"Orb.Instruction.i64/3","doc":"","ref":"Orb.Instruction.html#i64/3"},{"type":"function","title":"Orb.Instruction.local_get/2","doc":"","ref":"Orb.Instruction.html#local_get/2"},{"type":"function","title":"Orb.Instruction.local_set/2","doc":"","ref":"Orb.Instruction.html#local_set/2"},{"type":"function","title":"Orb.Instruction.local_set/3","doc":"","ref":"Orb.Instruction.html#local_set/3"},{"type":"function","title":"Orb.Instruction.local_tee/3","doc":"","ref":"Orb.Instruction.html#local_tee/3"},{"type":"function","title":"Orb.Instruction.memory_grow/1","doc":"","ref":"Orb.Instruction.html#memory_grow/1"},{"type":"function","title":"Orb.Instruction.memory_size/0","doc":"","ref":"Orb.Instruction.html#memory_size/0"},{"type":"function","title":"Orb.Instruction.memory_store/3","doc":"","ref":"Orb.Instruction.html#memory_store/3"},{"type":"function","title":"Orb.Instruction.new/2","doc":"","ref":"Orb.Instruction.html#new/2"},{"type":"function","title":"Orb.Instruction.new/3","doc":"","ref":"Orb.Instruction.html#new/3"},{"type":"function","title":"Orb.Instruction.typed_call/3","doc":"","ref":"Orb.Instruction.html#typed_call/3"},{"type":"function","title":"Orb.Instruction.typed_call/4","doc":"","ref":"Orb.Instruction.html#typed_call/4"},{"type":"function","title":"Orb.Instruction.wrap_constant!/2","doc":"","ref":"Orb.Instruction.html#wrap_constant!/2"},{"type":"module","title":"Orb.InstructionSequence","doc":"","ref":"Orb.InstructionSequence.html"},{"type":"function","title":"Orb.InstructionSequence.do_get_type/2","doc":"","ref":"Orb.InstructionSequence.html#do_get_type/2"},{"type":"function","title":"Orb.InstructionSequence.expand/1","doc":"","ref":"Orb.InstructionSequence.html#expand/1"},{"type":"function","title":"Orb.InstructionSequence.new/1","doc":"","ref":"Orb.InstructionSequence.html#new/1"},{"type":"function","title":"Orb.InstructionSequence.new/3","doc":"","ref":"Orb.InstructionSequence.html#new/3"},{"type":"behaviour","title":"Orb.Iterator","doc":"","ref":"Orb.Iterator.html"},{"type":"callback","title":"Orb.Iterator.next/1","doc":"","ref":"Orb.Iterator.html#c:next/1"},{"type":"callback","title":"Orb.Iterator.valid?/1","doc":"","ref":"Orb.Iterator.html#c:valid?/1"},{"type":"callback","title":"Orb.Iterator.value/1","doc":"","ref":"Orb.Iterator.html#c:value/1"},{"type":"type","title":"Orb.Iterator.var_ref/0","doc":"","ref":"Orb.Iterator.html#t:var_ref/0"},{"type":"module","title":"Orb.Leb","doc":"","ref":"Orb.Leb.html"},{"type":"function","title":"Orb.Leb.uleb128/1","doc":"","ref":"Orb.Leb.html#uleb128/1"},{"type":"module","title":"Orb.Loop.Branch","doc":"","ref":"Orb.Loop.Branch.html"},{"type":"module","title":"Orb.Memory","doc":"Work with memory.","ref":"Orb.Memory.html"},{"type":"function","title":"Orb.Memory.grow!/1","doc":"Increases the size of the memory instance by a specified number of pages. Each page is 64KiB.\n\nNote: only `0` and `1` are supported in today’s runtimes.\n\nIt returns the previous size of memory in pages if the operation was successful, or returns -1 if the operation failed.\n\nhttps://developer.mozilla.org/en-US/docs/WebAssembly/Reference/Memory/Grow\n\n```elixir\nMemory.grow!(1)\n```","ref":"Orb.Memory.html#grow!/1"},{"type":"macro","title":"Orb.Memory.initial_data!/2","doc":"Initializes data in memory. In Wat is `(data …)`","ref":"Orb.Memory.html#initial_data!/2"},{"type":"function","title":"Orb.Memory.load!/2","doc":"Load value of `type` from memory `address`.\n\n```elixir\nMemory.load!(I32, 0x100)\nMemory.load!(I32.U8, 0x100)\n```","ref":"Orb.Memory.html#load!/2"},{"type":"function","title":"Orb.Memory.page_byte_size/0","doc":"","ref":"Orb.Memory.html#page_byte_size/0"},{"type":"macro","title":"Orb.Memory.pages/1","doc":"Declare how many 64Kib pages of memory your module needs.\n\nCan be called multiple times, with each summed up.\n\nReturns the previous page count, which can be used as a start offset.","ref":"Orb.Memory.html#pages/1"},{"type":"function","title":"Orb.Memory.size/0","doc":"Returns the number of pages the memory instance currently has. Each page is 64KiB.\n\nTo increase the number of pages call `grow!/1`.\n\nhttps://developer.mozilla.org/en-US/docs/WebAssembly/Reference/Memory/Size\n\n```elixir\nMemory.size()\n```","ref":"Orb.Memory.html#size/0"},{"type":"function","title":"Orb.Memory.store!/3","doc":"Store `value` of `type` at memory `address`.\n\n```elixir\nMemory.store!(I32, 0x100, 42)\nMemory.store!(I32.U8, 0x100, ?a)\n```","ref":"Orb.Memory.html#store!/3"},{"type":"module","title":"Orb.Memory.Range","doc":"","ref":"Orb.Memory.Range.html"},{"type":"function","title":"Orb.Memory.Range.from/2","doc":"","ref":"Orb.Memory.Range.html#from/2"},{"type":"function","title":"Orb.Memory.Range.get_byte_length/1","doc":"","ref":"Orb.Memory.Range.html#get_byte_length/1"},{"type":"function","title":"Orb.Memory.Range.get_byte_offset/1","doc":"","ref":"Orb.Memory.Range.html#get_byte_offset/1"},{"type":"module","title":"Orb.MutRef","doc":"Use `Orb.DSL.mut!/1` to get a mutable reference to a global or local.","ref":"Orb.MutRef.html"},{"type":"function","title":"Orb.MutRef.from/1","doc":"","ref":"Orb.MutRef.html#from/1"},{"type":"function","title":"Orb.MutRef.store/2","doc":"","ref":"Orb.MutRef.html#store/2"},{"type":"module","title":"Orb.Numeric.Add","doc":"WebAssembly numeric `add` instruction:\n- `i32.add`\n- `i64.add`\n- `f32.add`\n- `f64.add`\n\n[MDN reference](https://developer.mozilla.org/en-US/docs/WebAssembly/Reference/Numeric/Addition)","ref":"Orb.Numeric.Add.html"},{"type":"function","title":"Orb.Numeric.Add.optimized/3","doc":"","ref":"Orb.Numeric.Add.html#optimized/3"},{"type":"module","title":"Orb.Numeric.Multiply","doc":"WebAssembly numeric `mul` instruction:\n- `i32.mul`\n- `i64.mul`\n- `f32.mul`\n- `f64.mul`\n\n[MDN reference](https://developer.mozilla.org/en-US/docs/WebAssembly/Reference/Numeric/Multiplication)","ref":"Orb.Numeric.Multiply.html"},{"type":"function","title":"Orb.Numeric.Multiply.optimized/3","doc":"","ref":"Orb.Numeric.Multiply.html#optimized/3"},{"type":"module","title":"Orb.Numeric.NotEqual","doc":"WebAssembly numeric `ne` instruction:\n- `i32.ne`\n- `i64.ne`\n- `f32.ne`\n- `f64.ne`\n\n[MDN reference](https://developer.mozilla.org/en-US/docs/WebAssembly/Reference/Numeric/Not_equal)","ref":"Orb.Numeric.NotEqual.html"},{"type":"function","title":"Orb.Numeric.NotEqual.optimized/3","doc":"","ref":"Orb.Numeric.NotEqual.html#optimized/3"},{"type":"module","title":"Orb.Numeric.Subtract","doc":"WebAssembly numeric `sub` instruction:\n- `i32.sub`\n- `i64.sub`\n- `f32.sub`\n- `f64.sub`\n\n[MDN reference](https://developer.mozilla.org/en-US/docs/WebAssembly/Reference/Numeric/Subtraction)","ref":"Orb.Numeric.Subtract.html"},{"type":"function","title":"Orb.Numeric.Subtract.optimized/3","doc":"","ref":"Orb.Numeric.Subtract.html#optimized/3"},{"type":"module","title":"Orb.S32.DSL","doc":"Signed 32-bit integer operators.","ref":"Orb.S32.DSL.html"},{"type":"function","title":"Orb.S32.DSL.//2","doc":"","ref":"Orb.S32.DSL.html#//2"},{"type":"function","title":"Orb.S32.DSL.</2","doc":"","ref":"Orb.S32.DSL.html#%3C/2"},{"type":"function","title":"Orb.S32.DSL.<=/2","doc":"","ref":"Orb.S32.DSL.html#%3C=/2"},{"type":"function","title":"Orb.S32.DSL.>/2","doc":"","ref":"Orb.S32.DSL.html#%3E/2"},{"type":"function","title":"Orb.S32.DSL.>=/2","doc":"","ref":"Orb.S32.DSL.html#%3E=/2"},{"type":"function","title":"Orb.S32.DSL.>>>/2","doc":"","ref":"Orb.S32.DSL.html#%3E%3E%3E/2"},{"type":"module","title":"Orb.Stack.Drop","doc":"","ref":"Orb.Stack.Drop.html"},{"type":"function","title":"Orb.Stack.Drop.new/1","doc":"","ref":"Orb.Stack.Drop.html#new/1"},{"type":"module","title":"Orb.Stack.Pop","doc":"","ref":"Orb.Stack.Pop.html"},{"type":"function","title":"Orb.Stack.Pop.new/1","doc":"","ref":"Orb.Stack.Pop.html#new/1"},{"type":"module","title":"Orb.Table","doc":"","ref":"Orb.Table.html"},{"type":"macro","title":"Orb.Table.allocate/1","doc":"","ref":"Orb.Table.html#allocate/1"},{"type":"function","title":"Orb.Table.call/1","doc":"","ref":"Orb.Table.html#call/1"},{"type":"function","title":"Orb.Table.call/2","doc":"","ref":"Orb.Table.html#call/2"},{"type":"function","title":"Orb.Table.call/3","doc":"","ref":"Orb.Table.html#call/3"},{"type":"function","title":"Orb.Table.call/4","doc":"","ref":"Orb.Table.html#call/4"},{"type":"function","title":"Orb.Table.call/5","doc":"","ref":"Orb.Table.html#call/5"},{"type":"function","title":"Orb.Table.call_indirect/2","doc":"","ref":"Orb.Table.html#call_indirect/2"},{"type":"function","title":"Orb.Table.call_indirect/3","doc":"","ref":"Orb.Table.html#call_indirect/3"},{"type":"function","title":"Orb.Table.call_indirect/4","doc":"","ref":"Orb.Table.html#call_indirect/4"},{"type":"function","title":"Orb.Table.call_indirect/5","doc":"","ref":"Orb.Table.html#call_indirect/5"},{"type":"function","title":"Orb.Table.call_indirect/6","doc":"","ref":"Orb.Table.html#call_indirect/6"},{"type":"function","title":"Orb.Table.do_elem/4","doc":"","ref":"Orb.Table.html#do_elem/4"},{"type":"macro","title":"Orb.Table.elem!/3","doc":"","ref":"Orb.Table.html#elem!/3"},{"type":"macro","title":"Orb.Table.fetch!/1","doc":"","ref":"Orb.Table.html#fetch!/1"},{"type":"macro","title":"Orb.Table.lookup!/1","doc":"","ref":"Orb.Table.html#lookup!/1"},{"type":"macro","title":"Orb.Table.lookup!/2","doc":"","ref":"Orb.Table.html#lookup!/2"},{"type":"module","title":"Orb.Table.CallIndirect","doc":"","ref":"Orb.Table.CallIndirect.html"},{"type":"behaviour","title":"Orb.Table.Type","doc":"","ref":"Orb.Table.Type.html"},{"type":"callback","title":"Orb.Table.Type.table_func_keys/0","doc":"","ref":"Orb.Table.Type.html#c:table_func_keys/0"},{"type":"callback","title":"Orb.Table.Type.type_name/0","doc":"","ref":"Orb.Table.Type.html#c:type_name/0"},{"type":"protocol","title":"Orb.ToWasm","doc":"","ref":"Orb.ToWasm.html"},{"type":"function","title":"Orb.ToWasm.to_wasm/2","doc":"Protocol for converting to WebAssembly binary format (.wasm)","ref":"Orb.ToWasm.html#to_wasm/2"},{"type":"type","title":"Orb.ToWasm.t/0","doc":"All the types that implement this protocol.","ref":"Orb.ToWasm.html#t:t/0"},{"type":"protocol","title":"Orb.ToWat","doc":"","ref":"Orb.ToWat.html"},{"type":"function","title":"Orb.ToWat.to_wat/2","doc":"Protocol for converting to WebAssembly text format (.wat)","ref":"Orb.ToWat.html#to_wat/2"},{"type":"type","title":"Orb.ToWat.t/0","doc":"All the types that implement this protocol.","ref":"Orb.ToWat.html#t:t/0"},{"type":"module","title":"Orb.ToWat.Helpers","doc":"","ref":"Orb.ToWat.Helpers.html"},{"type":"function","title":"Orb.ToWat.Helpers.do_type/1","doc":"","ref":"Orb.ToWat.Helpers.html#do_type/1"},{"type":"module","title":"Orb.TypeDefinition","doc":"","ref":"Orb.TypeDefinition.html"},{"type":"protocol","title":"Orb.TypeNarrowable","doc":"","ref":"Orb.TypeNarrowable.html"},{"type":"function","title":"Orb.TypeNarrowable.type_narrow_to/2","doc":"Protocol for narrowing types e.g. from Elixir.Integer to I32.","ref":"Orb.TypeNarrowable.html#type_narrow_to/2"},{"type":"type","title":"Orb.TypeNarrowable.t/0","doc":"All the types that implement this protocol.","ref":"Orb.TypeNarrowable.html#t:t/0"},{"type":"module","title":"Orb.U32.DSL","doc":"Unsigned 32-bit integer operators.\n\nSome operators like division have two variations in WebAssembly: signed and unsigned. By default math perform signed operations, but if you want unsigned math you can pass `U32` to `Orb.wasm/2` like so: `wasm U32 do`.","ref":"Orb.U32.DSL.html"},{"type":"function","title":"Orb.U32.DSL.//2","doc":"","ref":"Orb.U32.DSL.html#//2"},{"type":"function","title":"Orb.U32.DSL.</2","doc":"","ref":"Orb.U32.DSL.html#%3C/2"},{"type":"function","title":"Orb.U32.DSL.<=/2","doc":"","ref":"Orb.U32.DSL.html#%3C=/2"},{"type":"function","title":"Orb.U32.DSL.>/2","doc":"","ref":"Orb.U32.DSL.html#%3E/2"},{"type":"function","title":"Orb.U32.DSL.>=/2","doc":"","ref":"Orb.U32.DSL.html#%3E=/2"},{"type":"function","title":"Orb.U32.DSL.>>>/2","doc":"","ref":"Orb.U32.DSL.html#%3E%3E%3E/2"},{"type":"exception","title":"Orb.ModuleDefinition.FetchFuncError","doc":"","ref":"Orb.ModuleDefinition.FetchFuncError.html"},{"type":"exception","title":"Orb.TypeCheckError","doc":"","ref":"Orb.TypeCheckError.html"},{"type":"extras","title":"README","doc":"![Orb logo](orb-logo-orange.svg)\n\n# Orb: Write WebAssembly with Elixir\n\n[Docs](https://hexdocs.pm/orb)\n\nWrite WebAssembly with the power of Elixir as your compiler:\n\n- Use Elixir’s **module system** to break problems down and then compose them together.\n- Chain function calls together with the **pipe `|>` operator**.\n- Publish reusable code with the [**Hex package manager**](https://hex.pm).\n- **Write unit tests** using [Elixir’s built-in ExUnit](https://hexdocs.pm/ex_unit/ExUnit.html).\n- Reduce boilerplate with Elixir’s **powerful macro system**.\n- **Run dynamic Elixir code at compile time** e.g. talk to the rest of your Elixir application, or make network requests.\n- **Compile modules on-the-fly** e.g. use feature flags to conditionally compile code paths or enable particular WebAssembly instructions, creating a custom “tree shaken” WebAssembly module per user.","ref":"readme.html"},{"type":"extras","title":"Installation - README","doc":"Add `orb` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:orb, \"~> 0.0.33\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Example - README","doc":"```elixir\ndefmodule CalculateMean do\n  use Orb\n\n  global do\n    @tally 0\n    @count 0\n  end\n\n  defw insert(n: I32) do\n    @tally = @tally + n\n    @count = @count + 1\n  end\n\n  defw calculate_mean(), I32 do\n    @tally / @count\n  end\nend\n```\n\nThis can be converted to WebAssembly text format (wat):\n\n```elixir\nOrb.to_wat(CalculateMean)\n# \"\"\"\n# (module $CalculateMean\n#   (global $count (mut i32) (i32.const 0))\n#   (global $tally (mut i32) (i32.const 0))\n#   (func $insert (export \"insert\") (param $element i32)\n#     (i32.add (global.get $count) (i32.const 1))\n#     (global.set $count)\n#     (i32.add (global.get $tally) (local.get $element))\n#     (global.set $tally)\n#   )\n#   (func $calculate_mean (export \"calculate_mean\") (result i32)\n#     (i32.div_s (global.get $tally) (global.get $count))\n#   )\n# )\n# \"\"\"\n```\n\nYou can then execute it in Elixir with [OrbWasmtime](https://github.com/RoyalIcing/OrbWasmtime):\n\n```elixir\nalias OrbWasmtime.Instance\n\n# Run above example\ninst = Instance.run(CalculateMean)\nInstance.call(inst, :insert, 4)\nInstance.call(inst, :insert, 5)\nInstance.call(inst, :insert, 6)\nassert Instance.call(inst, :calculate_mean) == 5\n```\n\nNote there is [another excellent Elixir Wasmtime wrapper out there called Wasmex](https://github.com/tessi/wasmex), you may want to check that out too.","ref":"readme.html#example"},{"type":"extras","title":"Composing modules - README","doc":"You can compose modules together using `Orb.include/1`:\n\n```elixir\ndefmodule Math do\n  use Orb\n\n  defw square(n: I32), I32 do\n    n * n\n  end\nend\n\ndefmodule SomeOtherModule do\n  use Orb\n\n  Orb.include(Math)\n\n  defw magic(), I32 do\n    Math.square(3)\n  end\nend\n```","ref":"readme.html#composing-modules"},{"type":"extras","title":"Macros - README","doc":"TODO","ref":"readme.html#macros"},{"type":"extras","title":"Use cases - README","doc":"- Parsers\n- State machines\n- Formatters & string builders\n- HTTP endpoint that can be deployed agnostically to the server or edge.\n- Interactive UI controls\n- Write a HTML component and run it in:\n    - Phoenix LiveView & dead views\n    - In the browser using ` ` custom element\n- LiveView and its server rendering is a fantastic default, but the latency can be noticeable for certain UI interactions. With Orb you could use Elixir to write a WebAssembly module that then runs in the user’s browser.\n- Animation that runs fast in the browser and also works on the server\n- Code generators","ref":"readme.html#use-cases"},{"type":"extras","title":"Why WebAssembly? - README","doc":"- It runs on all of today’s major platforms: browser, server, edge, mobile, laptop, tablet, desktop.\n- Universal/isomorphic components (ones that run on the server and browser) are possible in React and Next.js, but they have many different flavours and can get pretty complex for a system that was meant to be declarative.\n- Like HTML and CSS it’s backwards compatible, which means WebAssembly you author today will be guaranteed to still work in a decade or longer.\n- It’s memory-safe and sandboxed. It can’t read memory outside of itself, only what has been explicitly passed into it. It can even be timeboxed to run for a maximum duration.\n- It’s fast.","ref":"readme.html#why-webassembly"},{"type":"extras","title":"Why develop Orb in Elixir? - README","doc":"Here are the reasons I chose to write Orb in Elixir.\n\n- Established language:\n    - Has package manager.\n    - Has composable modules with `alias` & `use`.\n    - Has syntax highlighting in IDEs, GitHub, and in highlighting libraries.\n    - Has language server with autocomplete.\n    - Has documentation system.\n    - Has unit test library.\n    - Has CI integration.\n    - Has linting.\n    - Integrates with native libraries in Rust and Zig.\n    - Has upcoming type system.\n- Established frameworks:\n    - Can integrate with Phoenix LiveView.\n    - Can connect to cloud, databases.\n    - Can integrate with Rust.\n- Community that is friendly and collaborative.\n- Can be extended with additional functions and macros:\n  - Unlike say C’s basic string-inserting preprocessor, Elixir is a full programming language without constraints.\n  - We can read files or the network and then generate code.\n  - You can create your own DSL. Want to enforce immutable-style programming? Want to add pattern matching? Design your own DSL on top of Orb for it.","ref":"readme.html#why-develop-orb-in-elixir"},{"type":"extras","title":"Orb Introduction","doc":"# Orb Introduction\n\n```elixir\nMix.install([\n  {:jason, \"~> 1.0\"},\n  :orb,\n  :orb_wasmtime\n])\n```","ref":"01-intro.html"},{"type":"extras","title":"Section - Orb Introduction","doc":"```elixir\nalias OrbWasmtime.Instance, as: Wasmtime\n```\n\n```elixir\ndefmodule Mean do\n  use Orb\n\n  I32.global(sum: 0, count: 0)\n\n  defw insert(n: I32) do\n    @count = @count + 1\n    @sum = @sum + n\n  end\n\n  defw calculate_mean(), I32 do\n    @sum / @count\n  end\nend\n```\n\n```elixir\na = Wasmtime.run(Mean)\nWasmtime.call(a, :insert, 3)\nWasmtime.call(a, :insert, 4)\nWasmtime.call(a, :insert, 5)\nWasmtime.call(a, :insert, 6)\nWasmtime.call(a, :insert, 7)\nWasmtime.call(a, :calculate_mean)\n```","ref":"01-intro.html#section"}],"content_type":"text/markdown"}